import CastorcitoForm from './components/Form/castorcito_form.js';

const fields = drupalSettings.field_components;
Object.entries(fields).forEach(([key, field]) => {
  const app = Vue.createApp({
    components: {
      'castorcito-form': CastorcitoForm
    },
    data() {
      const isUniqueComponent = field.components.length === 1 && field.cardinality === 1;
      const componentList = Object.entries(field.components).reduce((acc, [key, value]) => {
        acc[value] = drupalSettings.components[value].label;
        return acc;
      }, {});

      return {
        formData: [],
        showButtonAddComponent: true,
        componentList,
        isUniqueComponent
      };
    },
    mounted() {
      const textareas = document.querySelectorAll(`textarea[name^="${key}["]`);
      textareas.forEach((textarea) => {
        if (textarea.value) {
          try {
            let textareaJson = JSON.parse(textarea.value);
            this.setFormData('', textareaJson);
          } catch (error) {
            console.error(`Error parsing JSON in textarea: ${error.message}`);
          }
        }
        else {
          if (this.isUniqueComponent) {
            this.setFormData(field.components[0]);
          }
        }
      });
    },
    methods: {
      setFormData(type, storedModel = {}) {
        let component = {};
        if (type) {
          component = drupalSettings.components?.[type];
          if (!component) {
            console.warn(`Component ID ${type} was not found.`);
            return;
          }
        }

        //const fieldsSettings = JSON.parse(JSON.stringify(this.formData.fieldsSettings));
        const fieldsSettings = drupalSettings.components_fields_settings[storedModel.type || type];
        const model = type ? structuredClone(component) : this.orderFields(storedModel, fieldsSettings);
        const componentId = storedModel.type || type;

        this.formData.push({
          componentModel: model,
          fieldsSettings: fieldsSettings,
          fieldNameDrupal: key,
          componentId: componentId
        });

        if (field.cardinality !== -1 && field.cardinality <= this.formData.length) {
          this.handleToggleShowButton();
        }
      },
      orderFields(component, fieldsSettings) {
        if (!fieldsSettings?.order_fields || !component?.fields) {
          return component;
        }

        const orderedFields = {};
        fieldsSettings.order_fields.forEach((key) => {
          const field = component.fields[key];
          if (field) {
            if (field.type === 'container' && Array.isArray(field.items)) {
              field.items = field.items.map((item) => {
                const itemSettings = drupalSettings.components_fields_settings[item.type];
                return this.orderFields(item, itemSettings);
              });
            }

            orderedFields[key] = field;
          }
        });

        component.fields = orderedFields;
        return component;
      },
      addComponent(event) {
        const parent = event.target.parentElement;

        if (field.components.length === 1 && field.cardinality != 1) {
          this.setFormData(field.components[0]);
        }
        else {
          const selectElement = parent.querySelector(`#${key}_select_component`);

          if (selectElement.value === '') {
            selectElement.reportValidity();
            selectElement.focus();
          }
          else {
            this.setFormData(selectElement.value);
          }
        }

        if (field.cardinality === -1) {
          var addAnotherBoton = document.querySelector(`input[target_field_name="add-another-item-${field.field_name_class}"]`);
          var eventoMousedown = new MouseEvent('mousedown', {
            bubbles: true,
            cancelable: true,
            view: window,
          });

          addAnotherBoton.dispatchEvent(eventoMousedown);
        }
      },
      handleToggleShowButton() {
        this.showButtonAddComponent = !this.showButtonAddComponent;
      }
    }
  });

  app.mount(`#app_${key} .castorcito-form-container`);
});
