import castorcito_fields from '../castorcito_fields.js';

export default {
  components: {
    'castorcito-fields': castorcito_fields
  },
  props: [
    'fieldModel',
    'componentFieldSettings',
    'fieldBaseName',
    'fieldName'
  ],
  data() {
    const { components: allComponents, components_fields_settings: componentsFieldsSettings } = drupalSettings;
    const { min, max, add_button_label: addButtonLabel, delete_button_label: deleteButtonLabel, allowed_children: allowedChildren } = this.componentFieldSettings.settings;

    return {
      allComponents,
      componentsFieldsSettings,
      allowedChildren,
      min,
      max,
      addButtonLabel,
      deleteButtonLabel,
      selectedComponent: '',
      draggedItemIndex: null
    };
  },
  created() {
    if (!this.hasMultipleComponents && !this.fieldModel.items.length) {
      this.fieldModel.items.push(JSON.parse(JSON.stringify(this.allComponents[Object.keys(this.allowedChildren)[0]])));
    }
  },
  methods: {
    addItem() {
      const component = this.hasMultipleComponents ? this.selectedComponent : Object.keys(this.allowedChildren)[0];
      this.fieldModel.items.push(JSON.parse(JSON.stringify(this.allComponents[component])));
    },
    deleteItem: function(event, items, key) {
      items.splice(parseInt(key), 1);
    },
    formData(field) {
      return {
        componentModel: field,
        fieldsSettings: this.componentsFieldsSettings[field.type]
      };
    },
    labelDefault(item, i) {
      const order = i + 1;
      let label = 'Item - ' + order;
      if (this.componentFieldSettings.label !== item.label) {
        label = item.label;
      }

      return label;
    },
    activeToggle(event) {
      const element = event.target.parentElement;
      const content = element.querySelector('.castorcito-drag--content');
      if (content) {
        content.style.display = content.style.display === 'none' ? 'block' : 'none';
        element.hasAttribute('open') ? element.removeAttribute('open') : element.setAttribute('open', '');
      }
    },
    startDrag(event, item) {
      event.dataTransfer.dropEffect = 'move'
      event.dataTransfer.effectAllowed = 'move'
      //event.dataTransfer.setData('item_key', item);
      this.draggedItemIndex = item;
    },
    onDrop(event, i, items) {
      const itemKey = parseInt(this.draggedItemIndex, 10);
      if (isNaN(itemKey)) {
        console.error('Invalid item_key:', itemKey);
        return;
      }

      if (itemKey === i) return;

      const element = items.splice(itemKey, 1)[0];
      items.splice(i, 0, element);
    },
  },
  computed: {
    hasMultipleComponents() {
      return Object.keys(this.allowedChildren).length > 1;
    },
  },
  template: `
    <fieldset class="fieldset js-form-item form-item js-form-wrapper form-wrapper castorcito-field-container">
      <legend class="fieldset__legend fieldset__legend--visible">
        <span v-if="!componentFieldSettings.hide_label" class="fieldset__label">{{ componentFieldSettings.label }}</span>
      </legend>
      <div class="fieldset__wrapper">
        <div class="form-item__description">{{ componentFieldSettings.description }}</div>
        <div
          class="castorcito--drag"
          v-for="(item, i) in fieldModel.items"
          :key="i"
          @drop="onDrop($event, i, fieldModel.items)"
          @dragover.prevent
          @dragenter.prevent
        >
          <a v-if="fieldModel.items.length > 1" title="Drag to re-order" class="tabledrag-handle js-tabledrag-handle tabledrag-handle-y"></a>
          <div open class="claro-details" data-once="details" draggable="true" @dragstart="startDrag($event, i)">
            <div class="claro-details__summary" @click="activeToggle">
              <span v-text="labelDefault(item, i)"></span>
              <span class="castorcito--drag-changed"></span>
              <span class="claro-details__summary-summary"></span>
            </div>
            <div class="castorcito-container castorcito-drag--content">
              <castorcito-fields
                :formData="formData(item)"
                :fieldBaseName="fieldBaseName"
                :containerDelta="i + '_' + componentFieldSettings.name" />
                <button v-if="fieldModel.items.length > 1" type="button" class="button--small button--danger button" @click="deleteItem($event, fieldModel.items, i)">{{ deleteButtonLabel }}</button>
            </div>
          </div>
        </div>
        <div v-if="fieldModel.items.length < max" class="form-item castorcito-container--add-item">
          <div class="form-item--editor-format form-item form-type--select">
            <label v-if="hasMultipleComponents" :for="fieldName + '_add_item_select'" class="form-item__label visually-hidden">items</label>
            <select
              v-if="hasMultipleComponents"
              class="form-select form-element form-element--type-select"
              v-model="selectedComponent"
              :id="fieldName + '_add_item_select'">
              <option value="" disabled selected>Select an option</option>
              <option v-for="(child, key) in allowedChildren" :key="key" :value="key">{{ allComponents[child].label }}</option>
            </select>
            <button type="button" class="button button--action button--primary" @click="addItem">{{ addButtonLabel }}</button>
          </div>
        </div>
      </div>
    </fieldset>
  `
};
