import castorcito_fields from './castorcito_fields.js';

export default {
  components: {
    'castorcito-fields': castorcito_fields
  },
  props: ['formData', 'isUnique'],
  emits: ['toggle-show-button'],
  template: `
    <div v-if="!isUnique" class="fieldset__wrapper">
      <div class="castorcito--drag" v-for="(data, i) in formData" :key="i" @drop="onDrop($event, i, formData)" @dragover.prevent @dragenter.prevent>
        <a v-if="formData.length > 1" title="Drag to re-order" class="tabledrag-handle js-tabledrag-handle tabledrag-handle-y"></a>
        <div open class="claro-details" draggable="true" @dragstart="startDrag($event, i)">
          <div class="claro-details__summary" @click="activeToggle">
            <span>{{ data.componentModel.label }}</span>
            <span class="claro-details__summary-summary"></span>
          </div>
          <div class="castorcito-drag--content">
            <div tabindex="0" class="castorcito-form" @focusout="saveData(i)">
              <castorcito-fields
                :formData="data"
                :fieldBaseName="fieldBaseName(i)"
                :containerDelta="0"
              />
            </div>
            <div class="fieldset__wrapper">
              <button v-if="formData.length > 1" type="button" class="button--small button--danger button" @click="deleteComponent(i)">Delete</button>
            </div>
          </div>
        </div>
      </div>
    </div>
    <div v-else v-for="(data, i) in formData" :key="i" tabindex="0" class="castorcito-form" @focusout="saveData(i)">
      <castorcito-fields
        :formData="data"
        :fieldBaseName="fieldBaseName(i)"
        :containerDelta="0" />
    </div>
  `,
  methods: {
    fieldBaseName(delta) {
      const { fieldNameDrupal, componentId } = this.formData[delta];
      return `${fieldNameDrupal}_${delta}_${componentId}`;
    },
    activeToggle(event) {
      const element = event.target.parentElement;
      const content = element.querySelector('.castorcito-drag--content');
      if (content) {
        content.style.display = content.style.display === 'none' ? 'block' : 'none';
        element.hasAttribute('open') ? element.removeAttribute('open') : element.setAttribute('open', '');
      }
    },
    startDrag(event, item) {
      event.dataTransfer.dropEffect = 'move'
      event.dataTransfer.effectAllowed = 'move'
      event.dataTransfer.setData('item_key', item);
    },
    onDrop(event, list) {
      const itemKey = event.dataTransfer.getData('item_key');
      const element = this.formData.splice(itemKey, 1)[0];
      this.formData.splice(list, 0, element);

      // Saving updated data
      this.formData.forEach((_, index) => this.saveData(index));
    },
    deleteComponent(delta) {
      const { fieldNameDrupal } = this.formData[delta];

      // Clear corresponding textareas
      const textareas = document.querySelectorAll(`textarea[name^="${fieldNameDrupal}["]`);
      textareas.forEach(textarea => {
        textarea.value = '';
      });

      // Remove the component from the array
      this.formData.splice(delta, 1);

      // Saving updated data
      this.formData.forEach((_, index) => this.saveData(index));

      // Show button "Add Component".
      this.$emit('toggle-show-button');
    },
    saveData(delta) {
      const { componentModel, fieldNameDrupal } = this.formData[delta];
      const textareas = document.querySelectorAll(
        `textarea[name^="${fieldNameDrupal}[${delta}]"]`
      );

      textareas.forEach(textarea => {
        textarea.value = JSON.stringify(componentModel);
      });
    }
  }
};
